﻿/* ============================================================
* Author:       liubingsheng
* Time:         2016/11/29 14:52:25
* FileName:     RedisSystem
* Purpose:      Redis
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Utility
{
    /// <summary>
    /// DateTime扩展
    /// </summary>
    public static class DateTimeExtension
    {

        /// <summary>
        /// 某一天的一周时间列表
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static List<DateTime> DayLstOfWeek(this DateTime self)
        {
            int wd = (int)self.DayOfWeek;
            List<DateTime> week = new List<DateTime>();

            for (int i = 1 - wd; i < 8 - wd; i++)
            {
                week.Add(self.AddDays(i));
            }
            return week;
        }


        /// <summary>
        /// 今天是星期几,星期一是1，星期天是7
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static int DayOfWeekNum(this DateTime self)
        {
            int dayOfWeek = Convert.ToInt32(self.DayOfWeek) < 1 ? 7 : Convert.ToInt32(self.DayOfWeek);

            return dayOfWeek;
        }

        /// <summary>
        /// 本周第一天开始时
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static DateTime FirstDayOfWeek(this DateTime self)
        {
            DateTime firstDay = new DateTime(self.Year, self.Month, self.Day).AddDays(1 - self.DayOfWeekNum());
            return firstDay;
        }

        /// <summary>
        /// 本周最后一天时
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static DateTime LastDayOfWeek(this DateTime self)
        {
            DateTime firstDay = new DateTime(self.Year, self.Month, self.Day).AddDays(8 - self.DayOfWeekNum());
            return firstDay;
        }

        /// <summary>
        /// 本月第一天
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static DateTime FirstDayOfMonth(this DateTime self)
        {
            DateTime firstDay = self.AddDays(1 - self.Day);
            return firstDay;
        }

        /// <summary>
        /// 本月最后一天
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static DateTime LastDayOfMonth(this DateTime self)
        {
            DateTime firstDay = self.AddDays(1 - self.Day).AddMonths(1).AddDays(-1);
            return firstDay;
        }

        /// <summary>
        /// 是否是同一天
        /// </summary>
        /// <param name="self"></param>
        /// <param name="other"></param>
        /// <returns></returns>
        public static bool IsSameDay(this DateTime self, DateTime other)
        {
            return self.Date == other.Date;
        }

        /// <summary>
        /// 是否是同一周
        /// </summary>
        /// <param name="self"></param>
        /// <param name="other"></param>
        /// <returns></returns>
        public static bool IsSameWeek(this DateTime self, DateTime other)
        {
            return self.FirstDayOfWeek() == other.FirstDayOfWeek();
        }

        /// <summary>
        /// 是否是同一月
        /// </summary>
        /// <param name="self"></param>
        /// <param name="other"></param>
        /// <returns></returns>
        public static bool IsSameMonth(this DateTime self, DateTime other)
        {
            return self.Year == other.Year && self.Month == other.Month;
        }

        /// <summary>
        /// 获取周id
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static int WeekRangeOfYear(this DateTime self)
        {
            //DayOfYear+本年第一周少掉的天数-1，然后除以7，为本周id
            //如本年第一天为周五，本年第四天的周id为（4+5-2）/7+1=2

            int result = 0;
            DateTime firstDay = new DateTime(self.Year, 1, 1);
            result = (self.DayOfYear + firstDay.DayOfWeekNum() - 2) / 7 + 1;
            return result;
        }

        /// <summary>
        /// 转化为年月日格式字符串
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static string ToYMDString(this DateTime self)
        {
            return self.ToString("yyyyMMdd");
        }

        /// <summary>
        /// 转化成日期
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static Date ToDate(this DateTime self)
        {
            return new Date(self);
        }

        public static Week ToWeek(this DateTime self)
        {
            return new Week(self);
        }

        /// <summary>
        /// 获得总秒数
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static long TotalSecond(this DateTime self)
        {
            var span = self - DateTime.MinValue;
            double second = span.TotalSeconds;
            return (long)second;
        }

        /// <summary>
        /// 获得今日秒数
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static long TodaySecond(this DateTime self)
        {
            var span = self - self.Date;
            return (long)span.TotalSeconds;
        }

        /// <summary>
        /// 保留到小时的时间
        /// </summary>
        /// <returns></returns>
        public static DateTime HourTime(this DateTime self)
        {
            int hour = self.Hour;
            return self.Date.AddHours(hour);
        }

        /// <summary>
        /// 当前时间保留到小时
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static DateTime CurrentTimeRemainToHour()
        {
            int hour = DateTime.Now.Hour;

            return DateTime.Now.Date.AddHours(hour);
        }

        /// <summary>
        /// 时间间隔在多长span之内
        /// </summary>
        /// <param name="self"></param>
        /// <param name="other"></param>
        /// <param name="span"></param>
        /// <returns></returns>
        public static bool WithIn(this DateTime self, DateTime other, TimeSpan span)
        {
            if (self > other)
            {
                var temp = self - other;
                return temp <= span;
            }
            else
            {
                var temp = other - self;
                return temp <= span;
            }
        }

        public static DateTime GetDatetime(this string str)
        {
            DateTime result = DateTime.MinValue;

            string[] strArray = str.Split('T');
            if (strArray.Count() != 2)
            {
                return result;
            }

            string[] date = strArray[0].Split('-');
            if (date.Count() != 3)
            {
                return result;
            }
            string[] time = strArray[1].Split(':');
            if (time.Count() != 2)
            {
                return result;

            }

            int year = date[0].ToInt();
            int month = date[1].ToInt();
            int day = date[2].ToInt();
            int hour = time[0].ToInt();
            int minite = time[1].ToInt();

            result = new DateTime(year, month, day, hour, minite, 0);
            return result;

        }

        public static DateTime GetTime(this string str)
        {
            DateTime result = DateTime.MinValue;

            string[] time = str.Split(':');
            if (time.Count() != 2)
            {
                return result;
            }

            int hour = time[0].ToInt();
            int minite = time[1].ToInt();

            result = DateTime.MinValue.AddHours(hour).AddMinutes(minite);
            return result;

        }

        /// <summary>
        /// 通过字符串获取文本
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static DateTime GetDateTimeByString(string str, string format)
        {
            if (DateTime.TryParseExact(str, format, System.Globalization.CultureInfo.CurrentCulture, System.Globalization.DateTimeStyles.None, out DateTime result))
            {
                return result;
            }

            return DateTime.MinValue;
        }
    }
}

